package org.sap.utility;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.sap.customerService.BankSalesPerson;
import org.sap.customerService.CustomerDetail;
import org.sap.customerService.MessageHandler;
public class ComplaintSystem
{

    HashMap<String, ArrayList<CustomerQuery>> querMap;
    HashMap<String, CustomerDetail> customerDetail;
    HashMap<String, BankSalesPerson> bankDetails;
    String SMSContent;
    static Properties config;
    MessageHandler handler;
    String appKey;
    String appSecret;

    public ComplaintSystem(UploadDetails details) throws IOException
    {
        config = loadProperties("config.properties");
        String baseUrl = config.getProperty("baseUrl");
        appKey = config.getProperty("appKey"); //Replace with an App Key generated on the App Key area of the portal (Integrate - App Keys)
        appSecret = config.getProperty("appSecret"); //Replace with an App Secret generated on the App Key area of the portal (Integrate - App Keys)
        handler = new MessageHandler(baseUrl);
        
        customerDetail = new HashMap<String, CustomerDetail>();
        customerDetail = details.getCustomerDetailMap();
        bankDetails = new HashMap<String, BankSalesPerson>();
        bankDetails = details.getBankPersonDetails();
        loadComplaints();
        updateComplaint();
    }
    
    private Properties loadProperties(String filePath) throws IOException {
        Properties prop = new Properties();
        InputStream input = null;
        ClassLoader loader = Thread.currentThread().getContextClassLoader();

        try {
            input = loader.getResourceAsStream(filePath);
            // load a properties file
            prop.load(input);
        } catch (IOException ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return prop;
    }

    //Load all the complaints log to bank
    public void loadComplaints() throws IOException
    {
        querMap = new HashMap<String, ArrayList<CustomerQuery>>();
        FileReader fileRead = new FileReader(new File(getClass().getClassLoader().getResource("CustomerQuery.csv").getFile()));
        BufferedReader reader = new BufferedReader(fileRead);
        String record = null;
        while((record = reader.readLine()) != null)
        {
            ArrayList<CustomerQuery> list;
            String[] queryRecord = record.split("\\|");
            CustomerQuery query = new CustomerQuery();
            query.setCustomerId(queryRecord[0]);
            query.setQueryType(queryRecord[1]);
            query.setQueryContent(queryRecord[2]);
            if(querMap.containsKey(query.getCustomerId()))
            {
                list = querMap.get(query.getCustomerId());
                list.add(query);
            }
            else
            {
                list = new ArrayList<CustomerQuery>();
                list.add(query);
            }
            querMap.put(query.getCustomerId(), list);
        }
    }

    //Update customer by sending sms according to their query
    public void updateComplaint() throws IOException
    {
        Iterator<String> complaintIterator = querMap.keySet().iterator();
        while(complaintIterator.hasNext())
        {
            String id = complaintIterator.next();
            CustomerDetail detail = customerDetail.get(id);
            ArrayList<CustomerQuery> complaintList = querMap.get(id);
            for(int list = 0; list < complaintList.size(); list++)
            {
                CustomerQuery query = complaintList.get(list);
                if(query.getQueryType().equals("COMPLAINT"))
                {
                    BankSalesPerson person = getBankPerson("COMPLAINT");
                    SMSContent = "Hi " + detail.getName() + " on your complaint " + query.getQueryContent() + " Mr/Ms " + person.getName() + " will contact you Mobile number: " + person.getMobileNumber();
                    break;
                }
                if(query.getQueryType().equals("QUERY"))
                {
                    BankSalesPerson person = getBankPerson("QUERY");
                    SMSContent = "Hi " + detail.getName() + " on your query " + query.getQueryContent() + " Please connect to our sales person Mr/Ms " + person.getName() + " on " + person.getMobileNumber() + " Or our sales person will connect with you in next 3 working days.";
                    break;
                }
                if(query.getQueryType().equals("CRITICAL"))
                {
                    BankSalesPerson person = getBankPerson("CRITICAL");
                    SMSContent = "Hi " + detail.getName() + " on your issue " + query.getQueryContent() + " an appointment with manager is being fixed on " + getDateAndTime() + "";
                    break;
                }
            }
            handler.sendSMS(SMSContent, detail.getMobileNumber() + "", handler.getAccessToken(this.appKey, this.appSecret));
        }
    }

    public BankSalesPerson getBankPerson(String complaintType)
    {
        if(complaintType.equals("COMPLAINT"))
        {
            BankSalesPerson person = bankDetails.get("3");
            return person;
        }
        else
        {
            if(complaintType.equals("QUERY"))
            {
                BankSalesPerson person = bankDetails.get("2");
                return person;
            }
            else
            {
                if(complaintType.equals("CRITICAL"))
                {
                    BankSalesPerson person = bankDetails.get("1");
                    return person;
                }
            }
            return null;
        }
    }

    public String getDateAndTime()
    {
        int days = 1;
        long time = System.currentTimeMillis();
        Date sysDate = new Date(time);
        if(sysDate.getDay() > 5)
        {
            days = 3;
        }
        long newTime = System.currentTimeMillis() + (days * 1000 * 60 * 60 * 24);
        Date newCalculatedDate = new Date(newTime);
        String sysDateString = newCalculatedDate.toString();
        String[] dateSplit = sysDateString.split(" ");
        String appointmentDate = dateSplit[0] + " " + dateSplit[1] + " " + dateSplit[2] + " " + dateSplit[5];
        return appointmentDate;
    }

    public String getSMSContent()
    {
        return SMSContent;
    }

    public void setSMSContent(String sMSContent)
    {
        SMSContent = sMSContent;
    }


}